/*
 * Game.cpp
 *
 *  Created on: Aug 29, 2009
 *      Author: dportnoy
 */

#include "Game.hpp"

#include <Graphics/Graphics.hpp>
#include <Graphics/Font.hpp>

#include <sstream>

using std::vector;
using namespace std;

namespace {
	const float Tick = 0.01f;
}

#define UP 0
#define DOWN 1
#define LEFT 2
#define RIGHT 3
#define SPACE 4

// the vars below are a very hacky way to do this, but I don't have much time

bool keys[5];
int state; //state of the ball.

/*      1        5
 *    ---<--  --<--
 *	  |	B  |  |	B |
 * 	  |	B  |  |	B |
 * 	  |	B  |  |	B |4
 * 	 2|	B 0|  |6B |
 * 	  |	B /\  |	B /\
 * 	  |	B  |  \/B |
 * 	  |	B  |  |	B |
 * 	  |	B  |  |	B |
 *    -->------>--
 *        3
 *
 *  The game ends when the player gets to the end of 6
 */

float speed = 0.18f; //ball speed

int ticks;

bool won, lost;

Game::Game() : mouse_pos(make_vector(0.0f, 0.0f)), cam(), keystates(0),
				time_acc(0.0f), energy_acc(0) {
	quadric = gluNewQuadric();

	keys[UP] = false;
	keys[DOWN] = false;
	keys[LEFT] = false;
	keys[RIGHT] = false;
	keys[SPACE] = false;

	ticks = 0;

	won = lost = false;

	GLuint groundList = glGenLists(3);
	GLuint buildingList = groundList + 1;
	GLuint roadList = groundList + 2;
	GLuint ballList = groundList + 3;

	glNewList(buildingList, GL_COMPILE);
	glBegin(GL_QUADS);
		glColor3f(0.1f,0.1f,0.7f);

		/*
		glVertex3f( 3.0f, 0.0f, 3.0f);
		glVertex3f( 3.0f, 0.0f, -3.0f);
		glVertex3f( -3.0f, 0.0f, -3.0f);
		glVertex3f( -3.0f, 0.0f, 3.0f);
		*/

		//top
		glNormal3f( 0.0f, 1.0f, 0.0f);
		glVertex3f( 3.0f, 16.0f, 3.0f);
		glVertex3f( 3.0f, 16.0f, -3.0f);
		glVertex3f( -3.0f, 16.0f, -3.0f);
		glVertex3f( -3.0f, 16.0f, 3.0f);

		//sides

		glNormal3f( 1.0f, 0.0f, 0.0f);
		glVertex3f( 3.0f, 16.0f, 3.0f);
		glVertex3f( 3.0f, 16.0f, -3.0f);
		glVertex3f( 3.0f, 0.0f, -3.0f);
		glVertex3f( 3.0f, 0.0f, 3.0f);

		glNormal3f( 0.0f, 0.0f, -1.0f);
		glVertex3f( 3.0f, 16.0f, -3.0f);
		glVertex3f( -3.0f, 16.0f, -3.0f);
		glVertex3f( -3.0f, 0.0f, -3.0f);
		glVertex3f( 3.0f, 0.0f, -3.0f);

		glNormal3f( -1.0f, 0.0f, 0.0f);
		glVertex3f( -3.0f, 16.0f, -3.0f);
		glVertex3f( -3.0f, 16.0f, 3.0f);
		glVertex3f( -3.0f, 0.0f, 3.0f);
		glVertex3f( -3.0f, 0.0f, -3.0f);

		glNormal3f( 0.0f, 0.0f, 1.0f);
		glVertex3f( -3.0f, 16.0f, 3.0f);
		glVertex3f( 3.0f, 16.0f, 3.0f);
		glVertex3f( 3.0f, 0.0f, 3.0f);
		glVertex3f( -3.0f, 0.0f, 3.0f);
	glEnd();
	glEndList();

	glNewList(groundList, GL_COMPILE);
	glBegin(GL_QUADS);
		glColor3f(0.4f,0.4f,0.4f);
		glNormal3f( 0.0f, 1.0f, 0.0f);
		glVertex3f( 500.0f, 0.0f, 500.0f);
		glVertex3f( 500.0f, 0.0f, -500.0f);
		glVertex3f( -500.0f, 0.0f, -500.0f);
		glVertex3f( -500.0f, 0.0f, 500.0f);
	glEnd();
	glEndList();

	glNewList(roadList, GL_COMPILE);
	glBegin(GL_QUADS);
		glColor3f(0.54f,0.27f,0.07f);
		glNormal3f( 0.0f, 1.0f, 0.0f);
		glVertex3f( 8.0f, 0.0f, 500.0f);
		glVertex3f( 8.0f, 0.0f, -500.0f);
		glVertex3f( -8.0f, 0.0f, -500.0f);
		glVertex3f( -8.0f, 0.0f, 500.0f);
	glEnd();
	glEndList();

	glNewList(ballList, GL_COMPILE);
		gluSphere(quadric, 1.0f, 16, 10);
	glEndList();

	Object obj;

	for(int x=0; x<20; x++) {
		obj = Object();
		obj.list = buildingList;
		obj.translate = make_vector(-12.5f, -4.99f, 0.0f-x*10.0f);
		obj.rotate = make_vector(0.0f, 0.0f, 0.0f);

		obj.scale = make_vector(1.0f, rand() % 1000 / 1000.0f + 0.2f, 1.0f);	//scale from 1 to 3
		objects.push_back(obj);

		obj = Object();
		obj.list = buildingList;
		obj.translate = make_vector(12.5f, -4.99f, 0.0f-x*10.0f);
		obj.rotate = make_vector(0.0f, 0.0f, 0.0f);

		obj.scale = make_vector(1.0f, rand() % 1000 / 1000.0f + 0.2f, 1.0f);	//scale from 1 to 3
		objects.push_back(obj);
	}

	obj = Object();
	obj.list = groundList;
	obj.translate = make_vector(0.0f, -5.0f, 0.0f);
	obj.rotate = make_vector(0.0f, 0.0f, 0.0f);
	objects.push_back(obj);

	obj = Object();
	obj.list = roadList;
	obj.translate = make_vector(0.0f, -4.99f, 0.0f);
	obj.rotate = make_vector(0.0f, 0.0f, 0.0f);
	objects.push_back(obj);

	ball = Object();
	ball.list = ballList;
	ball.translate = make_vector(0.0f, 1.0f, -4.0f);
	ball.rotate = make_vector(0.0f, 0.0f, 0.0f);
}

Game::~Game() {
	gluDeleteQuadric(quadric);
}

void Game::draw() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();

	cam.applyTransform();

	vector<Object>::iterator it;

	for ( it=objects.begin() ; it < objects.end(); it++ ) {
		it->draw();
	}

	ball.draw();

	glDisable(GL_LIGHTING);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glScalef(1.0f / Graphics::aspectf(), 1.0f, 1.0f);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_LINE_SMOOTH);

	Graphics::FontRef font = Graphics::get_font("gentium.txf");
	assert(font.ref);

	std::stringstream out;

	if(ticks < 100)
		out << "3";
	else if(ticks < 200)
		out << "2";
	else if(ticks < 300)
		out << "1";
	else if(ticks < 400)
		out << "GO";

	glColor3f(0.0f, 1.0f, 0.0f);
	font->draw_center(out.str(), make_vector(0.0f, 0.0f), 0.6f);

	if(ticks < 400) {
		glColor3f(0.0f, 0.8f, 0.2f);

		out.str("");
		out << "You have to follow the ball closely.";
		font->draw_center(out.str(), make_vector(0.0f, -0.2f), 0.1f);

		out.str("");
		out << "If it gets to far from you, you will lose.";
		font->draw_center(out.str(), make_vector(0.0f, -0.3f), 0.1f);

		out.str("");
		out << "Use arrows keys to rotate and space bar to move forward.";
		font->draw_center(out.str(), make_vector(0.0f, -0.4f), 0.1f);
	}else if(won) {
		out << "Congratulations, you followed the ball to its destination.";
		glColor3f(0.0f, 1.0f, 0.0f);
		font->draw_center(out.str(), make_vector(0.0f, 0.0f), 0.1f);
	}else if(lost) {
		out << "Sorry, you let the ball get away.";
		glColor3f(1.0f, 0.0f, 0.0f);
		font->draw_center(out.str(), make_vector(0.0f, 0.0f), 0.1f);
	}

	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	glEnable(GL_LIGHTING);
}

void Game::tick() {
	ticks++;

	if(ticks < 400 || won || lost)
		return;

	if(state == 0) {
		ball.translate.z -= speed;
		if(ball.translate.z <= -206.0f)
			state++;
	}else if(state == 1) {
		ball.translate.x -= speed;
		if(ball.translate.x <= -20.0f)
			state++;
	}else if(state == 2) {
		ball.translate.z += speed;
		if(ball.translate.z >= 6.0f)
			state++;
	}else if(state == 3) {
		ball.translate.x += speed;
		if(ball.translate.x >= 20.0f)
			state++;
	}else if(state == 4) {
		ball.translate.z -= speed;
		if(ball.translate.z <= -206.0f)
			state++;
	}else if(state == 5) {
		ball.translate.x -= speed;
		if(ball.translate.x <= 0.0f)
			state++;
	}else if(state == 6) {
		ball.translate.z += speed;
		if(ball.translate.z >= 6.0f)
			won = true;
	}

	if(keys[UP]) {
		cam.rotate.x -= .5;
	}if(keys[DOWN]) {
		cam.rotate.x += .5;
	}if(keys[LEFT]) {
		cam.rotate.y -= .5;
	}if(keys[RIGHT]) {
		cam.rotate.y += .5;
	}if(keys[SPACE]) {
		cam.move(0.2f);
	}

	if(length(ball.translate-(-1*cam.translate)) > 18.0f) {
		/*cout << ball.translate.x << endl;
		cout << ball.translate.y << endl;
		cout << ball.translate.z << endl;
		cout << cam.translate.x << endl;
		cout << cam.translate.y << endl;
		cout << cam.translate.z << endl;*/
		lost = true;
	}
}

void Game::update(float elapsed) {
	time_acc += elapsed;
	while (time_acc > 0.0f) {
		time_acc -= Tick;
		tick();
	}
}

void Game::handle_event(SDL_Event const &event) {
	if (event.type == SDL_MOUSEMOTION) {
		mouse_pos.x = event.motion.x / float(Graphics::screen.x) * 2.0f * Graphics::aspectf() - Graphics::aspectf();
		mouse_pos.y = event.motion.y / float(Graphics::screen.y) *-2.0f + 1.0f;
	}
	if (event.type == SDL_KEYDOWN) {
		if(event.key.keysym.sym == SDLK_ESCAPE) {
			quit_screen(this);
		}else if(event.key.keysym.sym == SDLK_UP) {
			keys[UP] = true;
		}else if(event.key.keysym.sym == SDLK_DOWN) {
			keys[DOWN] = true;
		}else if(event.key.keysym.sym == SDLK_LEFT) {
			keys[LEFT] = true;
		}else if(event.key.keysym.sym == SDLK_RIGHT) {
			keys[RIGHT] = true;
		}else if(event.key.keysym.sym == SDLK_SPACE) {
			keys[SPACE] = true;
		}
	}

	if (event.type == SDL_KEYUP) {
		if(event.key.keysym.sym == SDLK_UP) {
			keys[UP] = false;
		}else if(event.key.keysym.sym == SDLK_DOWN) {
			keys[DOWN] = false;
		}else if(event.key.keysym.sym == SDLK_LEFT) {
			keys[LEFT] = false;
		}else if(event.key.keysym.sym == SDLK_RIGHT) {
			keys[RIGHT] = false;
		}else if(event.key.keysym.sym == SDLK_SPACE) {
			keys[SPACE] = false;
		}
	}
}

