#include <cstdlib>
#include <ctime>
#include <iostream>

#include <Basis/Screen.hpp>
#include <Basis/Log.hpp>
#include <Graphics/Graphics.hpp>

#include "Game.hpp"

void resizeGLScene(int width, int height)	{
	if (height==0)
		height=1;

	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

GLfloat LightAmbient[]= { 0.5f, 0.5f, 0.5f, 1.0f };

GLfloat LightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };

GLfloat LightPosition[]= { 0.0f, 20.0f, 2.0f, 1.0f };

void initGL() {
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);

	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);

	glLightfv(GL_LIGHT1, GL_POSITION, LightPosition);

	glEnable(GL_LIGHT1);

	glEnable(GL_LIGHTING);
}

int main(int argc, char **argv) {
	if (SDL_Init(SDL_INIT_TIMER) != 0) {
		LOG_ERROR("Could not initialize sdl: " << SDL_GetError());
		return 1;
	}

	//ask for a specific screen size:
	Graphics::screen.x = 800;
	Graphics::screen.y = 600;
	Graphics::fullscreen = false;

	if (!Graphics::init()) {
		LOG_ERROR("Could not initialize graphics, not continuing.");
		exit(1);
	}

	resizeGLScene(Graphics::screen.x, Graphics::screen.y);
	initGL();

	//we'll draw our own cursor
	SDL_ShowCursor(SDL_DISABLE);

	Game game;
	srand((unsigned)time(0));
	start_screen(&game);

	Graphics::deinit();

	SDL_Quit();

	return 0;
}
